/* REXX extension DLL to perform file copies & renames.           */

/* Copyright (C) 1997, 2003, David W. Noon.  All rights reserved. */
/* This code is released under the GNU General Public License.    */

/* This program is free software; you can redistribute it and/or modify      */
/* it under the terms of the GNU General Public License as published by      */
/* the Free Software Foundation; either version 2 of the License, or         */
/* (at your option) any later version.                                       */

/* This program is distributed in the hope that it will be useful,           */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
/* GNU General Public License for more details.                              */

/* You should have received a copy of the GNU General Public License         */
/* along with this program; if not, write to the Free Software               */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <stdlib.h>
#include <string.h>

#define INCL_NOPMAPI
#define INCL_DOSFILEMGR
#define INCL_DOSMODULEMGR
#define INCL_RXFUNC

#ifdef __EMX__
/* EMX+GCC uses its own OS/2 toolkit. */
#define INCL_REXXSAA
#include <os2emx.h>
#else
/* Other compilers use the vanilla OS/2 toolkit. */
#include <os2.h>
#include <rexxsaa.h>
#endif

#ifdef __WATCOMC__
/* This external global is provided by the Watcom CRTL. */
extern HMODULE __hmodule;

#if __WATCOMC__ >= 11
/* Define LibMain() for DLL start-up in Watxom 11.0 and later. */
APIRET APIENTRY LibMain(ULONG mod_handle, ULONG term_flag)
{
   return 1UL;
}
#endif
#else
/* Prototype for CRTL start-up routine. */
int _CRT_init(void);

/* This is a non-standard name, but it matches Watcom's global. */
static HMODULE __hmodule;

/* DLL start-up code for IBM, Borland and GCC compilers. */
unsigned long _DLL_InitTerm(unsigned long mod_handle, unsigned long term_flag)
{
   if (term_flag == 0)
   {
      /* Initialize DLL. */
      if (_CRT_init() != 0)
         return 0UL;

      /* Capture module handle for later. */
      __hmodule = (HMODULE) mod_handle;
   }
   else
   {
      /* Do nothing. */
   }

   return 1UL;
}
#endif

/* A list of all the function names published by this DLL. */
static const PCSZ func_name[] = {"FILERENAME", "FILECOPY"};

/* Prototypes to ensure linkage convention. */
RexxFunctionHandler FileAPIReg;
RexxFunctionHandler FileAPIDereg;
RexxFunctionHandler FileRename;
RexxFunctionHandler FileCopy;

/* This function is for use by SysLoadLibrary() only! */
ULONG FileAPIReg(PCSZ my_name, ULONG argc, RXSTRING argv[],
      PCSZ Queue_name, PRXSTRING Ret_str)
{
   char module_name[CCHMAXPATH];

   /* Obtain the name of this DLL. */
   DosQueryModuleName(__hmodule, sizeof module_name, module_name);

   /* Register both functions. */
   RexxRegisterFunctionDll(func_name[0], module_name, func_name[0]);
   RexxRegisterFunctionDll(func_name[1], module_name, func_name[1]);

   return 0UL;
}

/* This function is for use by SysDropLibrary() only! */
ULONG FileAPIDereg(PCSZ my_name, ULONG argc, RXSTRING argv[],
      PCSZ Queue_name, PRXSTRING Ret_str)
{
   /* De-register both functions. */
   RexxDeregisterFunction(func_name[0]);
   RexxDeregisterFunction(func_name[1]);

   return 0UL;
}

/* This function wraps the DosMove() API to rename files and/or */
/* move them around the directory structure within a partition. */

/* Usage:                                                       */
/*        ret_code = FileRename(Old_pathfile, New_pathfile)     */

/* The values for ret_code are the same as the APIRET values    */
/* returned by DosMove().                                       */
ULONG FileRename(PCSZ my_name, ULONG argc, RXSTRING argv[],
      PCSZ Queue_name, PRXSTRING Ret_str)
{
   char Old_filename[CCHMAXPATH], New_filename[CCHMAXPATH];

   /* Check that we have 2 arguments supplied and they are not too long */
   if (argc != 2UL ||
         argv[0].strlength >= CCHMAXPATH ||
         argv[1].strlength >= CCHMAXPATH)
      return 40UL; /* REX040 Invalid call to routine */

   /* Copy the 2 filenames to working storage & add trailing NULs */
   memcpy(Old_filename, argv[0].strptr, argv[0].strlength);
   Old_filename[argv[0].strlength] = '\0';
   memcpy(New_filename, argv[1].strptr, argv[1].strlength);
   New_filename[argv[1].strlength] = '\0';

   /* The DosMove() API peforms file renames. Pass its return code back. */
   Ret_str->strlength = strlen(_ultoa(DosMove(Old_filename, New_filename), Ret_str->strptr, 10));

   /* Return a correct result and let the REXX program handle any API error */
   return 0UL;
}

/* This function wraps the DosCopy() API to copy a file.        */

/* Usage:                                                       */
/*        ret_code = FileCopy(Input_file, Output_file)          */
/*        ret_code = FileCopy(Input_file, Output_file, 'O')     */
/*        ret_code = FileCopy(Input_file, Output_file, 'A')     */

/* The optional 3rd parameter indicates the action to be        */
/* taken when a file already exists with the target name:       */
/* 'O' means overwrite and 'A' means append.                    */

/* The values for ret_code are the same as the APIRET values    */
/* returned by DosCopy().                                       */
ULONG FileCopy(PCSZ my_name, ULONG argc, RXSTRING argv[],
      PCSZ Queue_name, PRXSTRING Ret_str)
{
   char Input_filename[CCHMAXPATH], Output_filename[CCHMAXPATH];
   ULONG option;

   /* Check that we have 2 or 3 arguments supplied and they are not too long */
   if (argc < 2UL || argc > 3UL ||
         argv[0].strlength >= CCHMAXPATH ||
         argv[1].strlength >= CCHMAXPATH)
      return 40UL; /* REX040 Invalid call to routine */

   /* Determine whether we want overwrite, append or fail. */
   if (argc == 2UL || argv[2].strlength == 0UL)
      option = 0UL;
   else
   {
      /* Use first letter of option, case insensitive, to determine */
      /* copy option: O = Overwrite; A = Append. */
      switch (argv[2].strptr[0])
      {
      case 'O':
      case 'o':
         /* Overwrite any existing file. */
         option = DCPY_EXISTING;
         break;
   
      case 'A':
      case 'a':
         /* Append to any existing file. */
         option = DCPY_APPEND;
         break;
   
      default:
         return 40UL; /* REX040 Invalid call to routine. */
         break;
      }
   }

   /* Copy the 2 filenames to working storage & add trailing NULs */
   memcpy(Input_filename, argv[0].strptr, argv[0].strlength);
   Input_filename[argv[0].strlength] = '\0';
   memcpy(Output_filename, argv[1].strptr, argv[1].strlength);
   Output_filename[argv[1].strlength] = '\0';

   /* The DosCopy() API peforms file copies. Pass its return code back. */
   Ret_str->strlength = strlen(_ultoa(DosCopy(Input_filename, Output_filename, option),
         Ret_str->strptr, 10));

   /* Return a correct result and let the REXX program handle any API error */
   return 0UL;
}
